<?php

namespace App\Http\Controllers;

use App\Traits\ApiResponseTrait;
// use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Tymon\JWTAuth\Facades\JWTAuth;
use Illuminate\Http\Request;
// use Tymon\JWTAuth\Facades\JWTAuth;
use Illuminate\Support\Facades\Auth;

class AuthController extends Controller
{
    //
    use ApiResponseTrait;

    public function login(Request $request){

        
        $currentGuardDB=Auth::getDefaultDriver();
        try {
            $rules = [
                "email" => "required|exists:".$currentGuardDB.",email",
                "password" => "required"
    
            ];
    
            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            }

            //login
            $credentials=$request->only(['email','password']);
            $token = Auth::guard()->attempt($credentials);
            if(!$token){
                return $this->error(0,"the email or password is incorrect");
            }


            $admin = Auth::guard()->user();
            $admin->token = $token;
            unset($admin->password);
            //return token
            return $this->returnData('admin', $admin,"success login");

        } catch (\Exception $ex) {
            return $this->error($ex->getCode(), $ex->getMessage());
        }
    }

    public function logout(Request $request){
        $token = $request->header('token');
        if($token){
            try{
                JWTAuth::setToken($token)->invalidate();
                return $this->success(0,'logged out successfully');
            }catch(\Tymon\JWTAuth\Exceptions\TokenInvalidException $e){
                return $this->error(1,'some thing went wrong!');    
            }
        }else{ 
            return $this->error(1,'some thing went wrong!');
        }
    }
}
