<?php

namespace App\Http\Controllers;

use App\Models\Admin;
use App\Models\Collection;
use App\Models\EndPoint;
use App\Traits\ApiResponseTrait;
use Illuminate\Http\Request;
use Exception;
use Illuminate\Database\QueryException;
use Tymon\JWTAuth\Facades\JWTAuth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\DB;

class CollectionController extends Controller
{
    use ApiResponseTrait;
    public function index()
    {
        $collections = Collection::get();
        return $this->returnData('collections', $collections, 'success');
    }
    public function getAll()
    {
        $collections = Collection::get();
        foreach($collections as $collection){
            $collection["admin"]=Admin::find($collection->admin_id);
            $collection["endpoint"]=EndPoint::find($collection["endpoint_id"]);
        }
        return $this->returnData('collections', $collections, 'success');
        
    }
    public function get(Request $request)
    {
        try {
            $collection = Collection::find($request->id);
            return $this->returnData('collection', $collection, 'success');
        } catch (Exception $e) {
            return $this->error(1, "this collection dosn't exist ! ");
        }
    }
    public function getEndpointCollections(Request $request)
    {
        try {
            $collections = Collection::where('endpoint_id', $request->endpoint_id)->get();
            foreach($collections as $collection){
                $collection["admin"]=Admin::find($collection->admin_id);
                $collection["endpoint"]=EndPoint::find($collection["endpoint_id"]);
            }
            
            return $this->returnData('collections', $collections, 'success');
            
        } catch (Exception $e) {
            return $this->error(1, "this collection dosn't exist ! ");
        }
    }
    public function create(Request $request)
    {

        try {
            $rules = [
                "admin_id" => "required",
                "endpoint_id" => "required",
                "amount" => "numeric|required",
                "collect_date" => "required",
                "collect_time" => "required",
            ];

            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                // Start a transaction
                DB::beginTransaction();

                try {
                    // Perform your database operations within the transaction
                    $note=($request->note==null)?"لا يوجد":$request->note;
                    DB::table('collections')
                        ->insert(
                            [
                                "admin_id" => $request->admin_id,
                                "endpoint_id" => $request->endpoint_id,
                                "amount" => $request->amount,
                                "collect_date" => $request->collect_date,
                                "collect_time" => $request->collect_time,
                                "note" => $note
                            ]
                        );

                    // Commit the transaction if all operations are successful
                    DB::commit();
                    return $this->success(0, "the collection has been created successfully!");
                } catch (\Exception $e) {
                    // An error occurred, rollback the transaction
                    DB::rollback();
                    return $this->error(1, "some thing went wrong while Creating this collection!");

                    // Handle the error or throw an exception
                    // ...
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while Creating this profile!");
        }
    }
}
