<?php

namespace App\Http\Controllers;

use App\Models\EndPoint;
use App\Traits\ApiResponseTrait;
use Exception;
use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class EndPointController extends Controller
{
    //
    use ApiResponseTrait;
    public function index(){
        $endpoints=EndPoint::get();
        return $this->returnData('endPoints',$endpoints,'success');
    }
    public function getAll(){
        $endpoints=EndPoint::get();
        return $this->returnData('endPoints',$endpoints,'success');
    }
    public function get(Request $request){
        try{
            $endPoint=EndPoint::find($request->id);
            return $this->returnData('endPoint',$endPoint,'success');
        }catch(Exception $e) {
            return $this->error(1,"this endPoint dosn't exist ! ");
        }
    }
    public function create(Request $request)
    {

        try {
            $rules = [
                "email" => "required",
                "password" => "required",
                "phone_number"=>"numeric",
                "address"=>"required",
                "place_name"=>"required"
            ];
    
            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            }else{
                $endpoint = new EndPoint();
                $endpoint->email =$request->email ;
                $endpoint->password = bcrypt($request->password);
                $endpoint->phone_number =$request->phone_number ;
                $endpoint->address =$request->address ;
                $endpoint->place_name =$request->place_name ;
                $success=$endpoint->save();
                if($success){
                    return $this->success(0,"the EndPoint profile has been created successfully!");
                }else{
                    return $this->error(1,"some thing went wrong while Creating this profile!");
                }
            }
            
        } catch (QueryException $e) {
            return $this->error(2,$e->getMessage());
        } catch (Exception $e) {
            return $this->error(1,"some thing went wrong while Creating this profile!");
        }
    }
    public function edit(Request $request)
    {

        try {
            $rules = [
                "id" => "required",
                "email" => "required",
                "phone_number"=>"numeric",
                "password"=>"required",
                "address"=>"required",
                "place_name"=>"required"
            ];
    
            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            }else{
                
                $endpoint=EndPoint::find($request->id);

                $endpoint->email =$request->email ;
                $endpoint->phone_number =$request->phone_number ;
                $endpoint->password = bcrypt($request->password);
                $endpoint->address =$request->address ;
                $endpoint->place_name =$request->place_name ;
                $success=$endpoint->save();

                if($success){
                    return $this->success(0,"the EndPoint profile has been Updated!");
                }else{
                    return $this->error(1,"some thing went wrong while Updating this profile!");
                }
            }
            
        } catch (QueryException $e) {
            return $this->error(2,"there is already an email or phone with same value!");
        } catch (Exception $e) {
            return $this->error(1,"some thing went wrong while Updating this profile!");
        }
    }

    public function delete(Request $request)
    {
        try {
            $rules = [
                'id' => "required",
            ];
            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $endpoint = EndPoint::find($request->id);
                $success=$endpoint->delete();
                if ($success) {
                    return $this->success(0,"the endpoint has been deleted successfully!");
                } else {
                    return $this->error(1, "some thing went wrong while deleting this endpoint!");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while deleting this endpoint!");
        }
    }

}
