<?php

namespace App\Http\Controllers;

use App\Models\EndPoint;
use App\Models\Package;
use App\Models\Subscription;
use App\Traits\ApiResponseTrait;
use Throwable;

use Illuminate\Http\Request;
use App\Models\Transaction;
use App\Models\User;
use App\Models\Wallet;
use Illuminate\Support\Facades\Validator;
use Illuminate\Database\QueryException;
use Exception;
use GrahamCampbell\ResultType\Success;
use Illuminate\Support\Facades\DB;
use PhpParser\Node\Expr\Cast\Double;

use function PHPUnit\Framework\isEmpty;

class PaymentTransactionsController extends Controller
{
    use ApiResponseTrait;
    public function index()
    {
        $transaction = Transaction::get();
        return $this->returnData('transaction', $transaction, 'success');
    }
    public function getAll()
    {
        $transaction = Transaction::get();
        foreach ($transaction as $trans) {
            $trans['user'] = User::find($trans->user_id);
            $trans['package'] = Package::find($trans->package_id);
            $trans['endpoint'] = EndPoint::find($trans->end_point_id);
        }
        return $this->returnData('transaction', $transaction, 'success');
    }

    public function get(Request $request)
    {
        try {
            $transaction = Transaction::find($request->id);
            $transaction['user'] = User::find($transaction->user_id);
            $transaction['package'] = Package::find($transaction->package_id);
            $transaction['endpoint'] = EndPoint::find($transaction->end_point_id);
            return $this->returnData('transaction', $transaction, 'success');
        } catch (Exception $e) {
            return $this->error(1, "this transaction dosn't exist ! ");
        }
    }
    public function getEndPointTransactions(Request $request)
    {
        try {
            $transactions = Transaction::where('end_point_id', $request->endpoint_id)->get();
            if ($transactions == null) {
                return $this->error(1, "there is no transactions for this endPoint! ");
            } else {
                $total = 0;
                foreach ($transactions as $transaction) {
                    $transaction['user'] = User::find($transaction->user_id);
                    $transaction['package'] = Package::find($transaction->package_id);
                    // $total = $total + (float)$transaction->payment_amount;
                    // $transaction['endpoint']=EndPoint::find($transaction->end_point_id);
                }
                // $transactions["total"] = $total;
                return $this->returnData('transactions', $transactions, 'success');
            }
        } catch (Exception $e) {
            return $this->error(1, "there is no transactions for this endPoint! ");
        }
    }
    public function getEndPointTotalTransactions(Request $request)
    {
        try {
            $transactions = Transaction::where('end_point_id', $request->endpoint_id)->get();
            if ($transactions == null) {
                return $this->error(1, "there is no transactions for this endPoint! ");
            } else {
                $total = 0;
                foreach ($transactions as $transaction) {
                    $total = $total + (float)$transaction->payment_amount;
                }
                $transactions["total"] = $total;
                return $this->returnData('transactions', $transactions, 'success');
            }
        } catch (Exception $e) {
            return $this->error(1, "there is no transactions for this endPoint! ");
        }
    }
    public function create(Request $request)
    {

        try {
            $rules = [
                'end_point_id' => 'required|numeric',
                'user_id' => 'required|numeric',
                'package_id' => 'required|numeric',
                'subscription_id' => 'required|numeric',
                'payment_amount' => 'required|numeric',
                'payment_method' => 'required'
            ];

            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {

                $transaction = new Transaction();
                $transaction->fill($request->all());
                $success1 = false;
                $success2 = false;
                $success3 = false;
                $subscription = Subscription::find($request->subscription_id); // Retrieve the model instance by its primary key

                if ($transaction->payment_method == 'wallet') {
                    $wallet = Wallet::where('user_id', $request->user_id)->first();

                    if ($request->payment_amount<=$wallet->amount) {

                        $payment_remain = $subscription->payment_remain; //geting remain
                        $payment_total = $subscription->payment_total; //getting total pay
                        $wallet_amount = $wallet->amount;//getting wallet amount

                        $payment_remain = ((Double)$payment_remain) - ((Double)($transaction->payment_amount)); //subtract from subscription remain
                        
                        $wallet_amount = ((Double)$wallet_amount) - ((Double)($transaction->payment_amount)); //subtract from wallet
                        $payment_total = ((Double)$payment_total) + ((Double)($transaction->payment_amount));


                        $subscription->payment_remain = $payment_remain;
                        $subscription->payment_total = $payment_total;
                        $wallet->amount = $wallet_amount;

                        $success1 = $transaction->save();
                        $success2 = $subscription->save();
                        $success3 = $wallet->save();
                    } else {
                        return $this->error(1, "رصيد المحفظة غير كافي !!");
                    }
                } else {
                     
                    $payment_remain = $subscription->payment_remain; //geting remain
                    $payment_total = $subscription->payment_total; //getting total pay

                    $payment_remain = ((Double)$payment_remain) - ((Double)($transaction->payment_amount)); //subtract from subscription remain
                    $payment_total = ((Double)$payment_total) + ((Double)($transaction->payment_amount));

                    $subscription->payment_remain = $payment_remain;
                    $subscription->payment_total = $payment_total;

                    $success1 = $transaction->save();
                    $success2 = $subscription->save();
                    $success3 = true;
                }


                if ($success1 && $success2 && $success3) {

                    $user = User::find($request->user_id);
                    $MessageController = new MassegesController();
                    $type = $request->package_id;
                    if ($type == 0) {
                        $type = "يومية";
                    } else if ($type == 1) {
                        $type = "شهرية";
                    } else {
                        $type = "سنوية";
                    }
                    $start = substr($request->start_date, 0, 10);
                    $end = substr($request->end_date, 0, 10);
                    $renew = ($request->auto_renewal == 0) ? "غير تلقائي" : "تلقائي";

                    try {
                        $MessageController->sendSmsMessage("عزيزي المشترك '$user->user_name' شكرا لقيامك بتسديد مبلغ '$request->payment_amount' من قيمة الإشتراك الخاص بك. اجمالي المبالغ المستحقة '$subscription->payment_remain'", $user->phone_number);
                    } catch (Exception $exception) {
                    }

                    return $this->success(0, "the Transaction has been created successfully!");
                } else {
                    return $this->error(1, "some thing went wrong while Proforming this transaction!");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while Proforming this transaction!");
        }
    }
    public function seen(Request $request)
    {
        try {
            $rules = [
                "id" => "required",
            ];
            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $transaction = Transaction::find($request->id);

                $transaction->seen = 1;
                $success = $transaction->save();
                if ($success) {
                    return $this->returnData("transaction", $transaction, "the transaction has been seen!");
                } else {
                    return $this->error(1, "some thing went wrong while see This transactions !");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while see this transactions !");
        }
    }
}
