<?php

namespace App\Http\Controllers;
use App\Models\Wallet;
use App\Models\EndPoint;
use App\Models\Extension;
use App\Models\Package;
use App\Traits\ApiResponseTrait;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Models\Subscription;
use App\Models\Transaction;
use App\Models\User;
use Illuminate\Support\Facades\Validator;
use Illuminate\Database\QueryException;
use Exception;

class SubscriptionController extends Controller
{
    use ApiResponseTrait;
    public function index()
    {
        $subscription = Subscription::get();
        return $this->returnData('subscription', $subscription, 'success');
    }
    public function getAll()
    {
        $subscription = Subscription::get();
        foreach ($subscription as $sub) {
            $sub['user'] = User::find($sub->user_id);
            $sub['package'] = Package::find($sub->package_id);
            $sub['wallet'] = Wallet::where('user_id', $sub->user_id)->first();
            $sub['extend'] = Extension::where('subscription_id', $sub->id)->get();
        }
        return $this->returnData('subscription', $subscription, 'success');
    }
    public function get(Request $request)
    {
        try {
            $subscription = Subscription::find($request->id);
            $subscription['user'] = User::find($subscription->user_id);
            $subscription['package'] = Package::find($subscription->package_id);
            $subscription['payments'] = Transaction::where('subscription_id', $request->id)->get();
            $subscription['wallet'] = Wallet::where('user_id', $subscription->user_id)->first();
            $subscription['extend'] = Extension::where('subscription_id', $subscription->id)->get();

            foreach ($subscription['payments'] as $payment) {
                $payment['end_point'] = EndPoint::find($payment->end_point_id);
            }
            // $subscription['payments']["endpoint"]=EndPoint::find(13);
            // $subscription['payments']["package"]=Package::find($subscription['payments']["package_id"]);

            return $this->returnData('subscription', $subscription, 'success');
        } catch (Exception $e) {
            return $this->error(1, "this subscription dosn't exist ! ");
        }
    }
    public function getByUserId($id)
    {
        try {
            $subscription = Subscription::where('user_id', $id)->first();
            $subscription['user'] = User::find($subscription->user_id);
            $subscription['package'] = Package::find($subscription->package_id);
            $subscription['payments'] = Transaction::where('subscription_id', $subscription->id)->get();
            $subscription['wallet'] = Wallet::where('user_id', $subscription->user_id)->first();

            foreach ($subscription['payments'] as $payment) {
                $payment['end_point'] = EndPoint::find($payment->end_point_id);
            }
            return $this->returnData('subscription', $subscription, 'success');
        } catch (Exception $e) {
            return $this->error(1, "this subscription dosn't exist ! ");
        }
    }
    public function create(Request $request)
    {

        try {
            $rules = [
                'user_id' => "required",
                'package_id' => "required|numeric",
                'start_date' => "required",
                'end_date' => "required",
                'auto_renewal' => "required|numeric",
                'state' => "required|numeric",
                'price' => "required|numeric",
                'speed' => "required|numeric",
                'payment_remain' => "required|numeric",
                'payment_total' => "required|numeric",
            ];

            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $subscription = new Subscription();
                $subscription->fill($request->all());
                $remain = $subscription->payment_remain;
                $success = $subscription->save();
                $subscription["id"] = $subscription->id;
                
                $wallet = new Wallet();
                $wallet->user_id = $subscription->user_id;
                $wallet->amount = $request->walletAmount;
                $wallet->status=1;
                $success=$wallet->save();
                
                if ($success) {
                    $user = User::find($request->user_id);
                    $MessageController = new MassegesController();
                    $type = $request->package_id;
                    if ($type == 0) {
                        $type = "يومية";
                    } else if ($type == 1) {
                        $type = "شهرية";
                    } else {
                        $type = "سنوية";
                    }
                    $start = substr($request->start_date, 0, 10);
                    $end = substr($request->end_date, 0, 10);
                    $renew = ($request->auto_renewal == 0) ? "غير تلقائي" : "تلقائي";

                    try {
                        $name=$user->user_name;
                        $speed=$request->speed;
                        $price=$request->price;
                        $MessageController->sendSmsMessage("عزيزي المشترك ' $name ' شكرا لانضمامك لشبكة نيو برايت نت لقد إشتركت في باقة ' $type' بسرعة ' $speed ' وثمن الإشتراك '$price' و تبدأ من تاريخ '$start' إلى تاريخ '$end' ونوع التجديد '$renew' وإجمالي المبالغ المستحقة '$remain' . لتسديد فواتيرك يرجى التوجه لسوبر ماركت بيت المونة بجانب البنك الاسلامي بإدراة يونس حنتولي", $user->phone_number);
                    } catch (Exception $exception) {
                    }


                    return $this->returnData("subscription", $subscription, "the Subscription has been created successfully!");
                } else {
                    return $this->error(1, "the user already have Subscription!");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, "the user already have Subscription!");
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while Creating this Subscription!");
        }
    }
    public function delete(Request $request)
    {
        try {
            $rules = [
                'id' => "required",
            ];
            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $subscription = Subscription::find($request->id);
                $extension = Extension::where("subscription_id", $request->id)->first();
                if ($extension) {
                    $extension->delete();
                }
                $user = User::find($subscription->user_id);

                $success = $subscription->delete();
                if ($success) {


                    $MessageController = new MassegesController();


                    try {
                        $MessageController->sendSmsMessage("عزيزي المشترك '$user->user_name' في شبكة نيو برايت نت 
                        نود إعلامك أنه تم إلغاء و حذف الإشتراك الخاص بك .", $user->phone_number);
                    } catch (Exception $exception) {
                    }


                    return $this->success(0, "the Subscription has been deleted successfully!");
                } else {
                    return $this->error(1, "some thing went wrong while deleting this Subscription!");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while deleting this Subscription!");
        }
    }
    public function updateState(Request $request)
    {
        try {
            $rules = [
                "id" => "required",
                'state' => "required|numeric",
            ];

            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $subscription = Subscription::find($request->id);
                $subscription->state = $request->state;
                $subscription->seen = 0;



                $extension = Extension::where("subscription_id", $request->id)->first();
                if ($extension) {
                    $extension->delete();
                }

                $success = $subscription->save();
                if ($success) {
                    $user = User::find($subscription->user_id);
                    $MessageController = new MassegesController();
                    $msg = ($request->state == 0) ? "إلغاء" : "";
                    try {
                        $MessageController->sendSmsMessage("عزيزي المشترك '$user->user_name' في شبكة نيو برايت نت  نود إعلامك أنه تم $msg تفعيل الإشتراك الخاص بك .", $user->phone_number);
                    } catch (Exception $exception) {
                    }

                    return $this->returnData("subscription", $subscription, "the Subscription state has been updated successfully!");
                } else {
                    return $this->error(1, "some thing went wrong while Updating The subscription state !");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while Updating The subscription state !");
        }
    }
    public function updateRenual(Request $request)
    {
        try {
            $rules = [
                "id" => "required",
                'auto_renewal' => "required|numeric",
            ];

            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $subscription = Subscription::find($request->id);
                $subscription->auto_renewal = $request->auto_renewal;
                $subscription->seen = 0;
                $remain = $subscription->payment_remain;
                $success = $subscription->save();
                if ($success) {

                    $renew = ($request->auto_renewal == 0) ? "غير تلقائي" : "تلقائي";
                    $user = User::find($subscription->user_id);
                    $MessageController = new MassegesController();

                    try {
                        $MessageController->sendSmsMessage("عزيزي المشترك '$user->user_name' في شبكة نيو برايت نت  نود إعلامك أنه تم تعديل نوع تجديد الإشتراك الخاص بك الى '$renew' وإجمالي المبالغ المستحقة '$remain' .", $user->phone_number);
                    } catch (Exception $exception) {
                    }

                    return $this->returnData("subscription", $subscription, "the Subscription auto renewal has been updated successfully!");
                } else {
                    return $this->error(1, "some thing went wrong while Updating The subscription auto renewal !");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while Updating The subscription auto renewal !");
        }
    }

    public function renewSubscription(Request $request)
    {
        try {
            $rules = [
                "id" => "required",
                'price' => "required|numeric",
                'speed' => "required|numeric",
                'package_id' => "required|numeric",
                'auto_renewal' => "required|numeric",
                'start_date' => "required",
                'end_date' => "required",
            ];

            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $subscription = Subscription::find($request->id);

                $subscription->price = $request->price;
                $subscription->speed = $request->speed;
                $subscription->package_id = $request->package_id;
                $subscription->auto_renewal = $request->auto_renewal;
                $start=Carbon::parse($request->start_date)->toDateString().' 00:00:00.000';
                $end=Carbon::parse($request->end_date)->toDateString().' 00:00:00.000';
                $subscription->start_date = $start;
                $subscription->end_date = $end;
                $subscription->state = 1;
                $subscription->seen = 0;

                $subscription->payment_remain = ((Double)$subscription->payment_remain) + ((Double)$request->price);
                $remain = $subscription->payment_remain;
                $success = $subscription->save();

                $extension = Extension::where("subscription_id", $request->id)->first();
                if ($extension) {
                    $extension->delete();
                }

                if ($success) {

                    $user = User::find($subscription->user_id);
                    $MessageController = new MassegesController();
                    $type = $request->package_id;
                    if ($type == 0) {
                        $type = "يومية";
                    } else if ($type == 1) {
                        $type = "شهرية";
                    } else {
                        $type = "سنوية";
                    }
                    $start = substr($request->start_date, 0, 10);
                    $end = substr($request->end_date, 0, 10);
                    $renew = ($request->auto_renewal == 0) ? "غير تلقائي" : "تلقائي";

                    try {
                        $MessageController->sendSmsMessage("عزيزي المشترك '$user->user_name' شكرا لتجديد الباقه الخاصه بك في شبكة نيو برايت نت لقد تم التجديد في باقة '$type' بسرعة '$request->speed'  وثمن الإشتراك '$request->price' و تبدأمن تاريخ '$start' إلى تاريخ '$end' ونوع التجديد '$renew' وإجمالي المبالغ المستحقة '$remain' . ", $user->phone_number);
                    } catch (Exception $exception) {
                    }


                    return $this->returnData("subscription", $subscription, "the Subscription renewal has been updated successfully!");
                } else {
                    return $this->error(1, "some thing went wrong while Updating The subscription renewal !");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while Updating The subscription renewal !");
        }
    }
    public function edit(Request $request)
    {
        try {
            $rules = [
                "id" => "required",
                'price' => "required|numeric",
                'speed' => "required|numeric",
                'package_id' => "required|numeric",
                'payment_remain' => "required|numeric",
                'wallet' => "required|numeric",
                'auto_renewal' => "required",
                'start_date' => "required",
                'end_date' => "required",
            ];
            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $subscription = Subscription::find($request->id);

                $subscription->price = $request->price;
                $subscription->speed = $request->speed;
                $subscription->payment_remain = ((float)$request->payment_remain) + ((float)$request->price);
                $subscription->package_id = $request->package_id;
                $subscription->auto_renewal = $request->auto_renewal;
                $subscription->start_date = $request->start_date;
                $subscription->end_date = $request->end_date;
                $subscription->seen = 0;
                $extension = Extension::where("subscription_id", $request->id)->first();
                if ($extension) {
                    $extension->delete();
                }


                $user = User::find($subscription->user_id);

                $wallet = Wallet::where('user_id', $user->id)->first();
                $wallet->amount = $request->wallet;

                $success1 = $subscription->save();
                $success2 = $wallet->save();

                if ($success1 && $success2) {

                    return $this->returnData("subscription", $subscription, "the Subscription has been Edited successfully!");
                } else {
                    return $this->error(1, "some thing went wrong while Updating The subscription !");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while Updating The subscription !");
        }
    }
    public function seen(Request $request)
    {
        try {
            $rules = [
                "id" => "required",
            ];
            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $subscription = Subscription::find($request->id);

                $subscription->seen = 1;
                $success = $subscription->save();
                if ($success) {
                    return $this->returnData("subscription", $subscription, "the Subscription has been seen!");
                } else {
                    return $this->error(1, "some thing went wrong while see This subscription !");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while see this subscription !");
        }
    }
}
