<?php

namespace App\Http\Controllers;

use App\Models\EndPoint;
use App\Models\Package;
use App\Models\Subscription;
use App\Models\Transaction;
use App\Models\User;
use App\Models\Wallet;
use App\Traits\ApiResponseTrait;
use Exception;
use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

use function PHPUnit\Framework\isEmpty;

class UserController extends Controller
{
    use ApiResponseTrait;
    public function index()
    {
        $users = User::get();
        return $this->returnData('Users', $users, 'success');
    }
    public function getAll()
    {
        $users = User::get();
        return $this->returnData('Users', $users, 'success');
    }
    public function get(Request $request)
    {
        try {
            $user = User::find($request->id);
            return $this->returnData('User', $user, 'success');
        } catch (Exception $e) {
            return $this->error(1, "this user dosn't exist ! ");
        }
    }
    public function getBySearch(Request $request)
    {
        try {

            if ($request->phone_number) {
                $userId = User::where("phone_number", $request->phone_number)->first()->id;
                $subscription = Subscription::where('user_id', $userId)->first();
                $subscription['user'] = User::find($subscription->user_id);
                $subscription['package'] = Package::find($subscription->package_id);
                $subscription['payments'] = Transaction::where('subscription_id', $subscription->id)->get();
                $subscription['wallet'] = Wallet::where('user_id', $subscription->user_id)->first();
                foreach ($subscription['payments'] as $payment) {
                    $payment['end_point'] = EndPoint::find($payment->end_point_id);
                }
                return $this->returnData('subscription', $subscription, 'success');
            } else {

                $users = User::where("phone_number", $request->text)
                    ->orWhere('user_name', 'like', '%' . $request->text . '%')->get();
                if ($users->isEmpty()) {
                    return $this->error(1, "there is no users with this name or phone! ");
                }
                return $this->returnData('users', $users, 'success');
            }
        } catch (Exception $e) {
            return $this->error(1, "this user dosn't exist ! ");
        }
    }
    public function create(Request $request)
    {


        try {
            $rules = [
                "user_name" => "required",
                "phone_number" => "required|numeric",
                "address" => "required"
            ];

            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $exicst = User::where('phone_number', $request->phone_number)->first();
                if ($exicst) {
                    return $this->returnData('user', $exicst, 'the User profile already exist!');
                } else {
                    $user = new User();
                    $user->user_name = $request->user_name;
                    $user->phone_number = $request->phone_number;
                    $user->address = $request->address;
                    $success = $user->save();
                    $user["id"] = $user->id;
                    if ($success) {
                        return $this->returnData('user', $user, 'the User profile has been created successfully!');
                    } else {
                        return $this->error(1, "some thing went wrong while Creating this profile!");
                    }
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, "the user profile already exist");
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while Creating this profile!");
        }
    }

    public function edit(Request $request)
    {


        try {
            $rules = [
                "id" => "required",
                "user_name" => "required",
                "phone_number" => "required|numeric",
                "address" => "required"
            ];

            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $user = User::find($request->id);

                $user->user_name = $request->user_name;
                $user->phone_number = $request->phone_number;
                $user->address = $request->address;
                $success = $user->save();

                if ($success) {
                    return $this->success(0, 'the User profile has been Updated successfully!');
                } else {
                    return $this->error(1, "some thing went wrong while Updated this profile!");
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, "there is already phone with same value!");
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while Creating this profile!");
        }
    }

    public function delete(Request $request)
    {
        try {
            $rules = [
                'id' => "required",
            ];
            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                $code = $this->returnCodeAccordingToInput($validator);
                return $this->returnValidationError($code, $validator);
            } else {
                $user = User::find($request->id);
                $subscription = Subscription::where('user_id', $user->id)->exists();
                if ($subscription) {
                    return $this->error(1, "you can't delete this user he have a subscription!");
                } else {
                    $success = $user->delete();
                    if ($success) {
                        return $this->success(0, "the user has been deleted successfully!");
                    } else {
                        return $this->error(1, "some thing went wrong while deleting this user!");
                    }
                }
            }
        } catch (QueryException $e) {
            return $this->error(2, $e->getMessage());
        } catch (Exception $e) {
            return $this->error(1, "some thing went wrong while deleting this user!");
        }
    }
}
