<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Models\Wallet;
use App\Traits\ApiResponseTrait;
use Exception;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;


class WalletController extends Controller
{
    use ApiResponseTrait;
    
    function create(request $request){
        $rules = [
            'user_id' => 'required|numeric',
            'amount' => 'required|numeric'
        ];

        $validator = Validator::make($request->all(), $rules);

        if ($validator->fails()) {
            $code = $this->returnCodeAccordingToInput($validator);
            return $this->returnValidationError($code, $validator);
        } else {
            $wallet = new Wallet();
            $wallet->user_id = $request->user_id;
            $wallet->amount = $request->amount;
            $wallet->status=1;
            $success=$wallet->save();
            if($success){
                return $this->returnData('wallet',$wallet,'the wallet has been created successfully!');
            }else{
                return $this->error(1,"some thing went wrong while Creating this wallet!"); 
            }
        }
    }
    function getWalletBalance(request $request){
        $rules = [
            'user_id' => 'required|numeric',
        ];

        $validator = Validator::make($request->all(), $rules);

        if ($validator->fails()) {
            $code = $this->returnCodeAccordingToInput($validator);
            return $this->returnValidationError($code, $validator);
        } else {
            $wallet = Wallet::where('user_id',$request->user_id)->get();
            return $this->returnData('balance', $wallet->amount, 'success');

        }
    }
    function charge(request $request){
        $rules = [
            'user_id' => 'required|numeric',
            'amount' => 'required|numeric',
        ];

        $validator = Validator::make($request->all(), $rules);

        if ($validator->fails()) {
            $code = $this->returnCodeAccordingToInput($validator);
            return $this->returnValidationError($code, $validator);
        } else {
            $wallet = Wallet::where('user_id',$request->user_id)->first();
            $wallet->amount= ((Double)$wallet->amount)+((Double)$request->amount);
            $success=$wallet->save();
            if($success){
                
                $user = User::find($request->user_id);
                $MessageController = new MassegesController();
                $type = $request->package_id;
                if ($type == 0) {
                    $type = "يومية";
                } else if ($type == 1) {
                    $type = "شهرية";
                } else {
                    $type = "سنوية";
                }
                $start = substr($request->start_date, 0, 10);
                $end = substr($request->end_date, 0, 10);
                $renew = ($request->auto_renewal == 1) ? "غير تلقائي" : "تلقائي";

                // try {
                //     $MessageController->sendSmsMessage("عزيزي المشترك '$user->user_name' في شبكة نيو برايت نت  نود إعلامك أنه تم شحن رصيد المحفظة الخاص بك بمبلغ قدره '$request->amount' وإجمالي رصيدك هو '$wallet->amount' .", $user->phone_number);
                // } catch (Exception $exception) {
                // }
                return $this->returnData('balance', $wallet->amount, 'success');
            }else{
                return $this->error(1,"some thing went wrong while charging this wallet!"); 
            }

        }
    }
}
