<?php

use App\Http\Controllers\AdminController;
use App\Http\Controllers\AuthController;
use App\Http\Controllers\CollectionController;
use App\Http\Controllers\EndPointController;
use App\Http\Controllers\ExtensionContreoller;
use App\Http\Controllers\PaymentTransactionsController;
use App\Http\Controllers\SubscriptionController;
use App\Http\Controllers\SubscriptionLogController;
use App\Http\Controllers\UserController;
use App\Http\Controllers\WalletController;
use App\Http\Controllers\MassegesController;
use App\Models\EndPoint;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

//authinticated login and registeration for admin and  

Route::get('test', [AdminController::class, 'test']);

Route::group(['middleware' => ['api', 'CheckApiPassword', 'ChangeLanguage']], function () {
    /////////////////// Admin Apis
    Route::group(['prefix' => 'admin'], function () {
        Route::post('login', [AuthController::class, 'login'])->middleware('AssignGuard:admins');
        Route::post('logout', [AuthController::class, 'logout'])->middleware('AssignGuard:admins');


        Route::group(['middleware' => ['AssignGuard:admins', 'CheckValidToken']], function () {
            //controlling the admins operations
            Route::get('get-all-admins', [AdminController::class, 'getAll']);
            Route::get('get-admin', [AdminController::class, 'get']);
            Route::post('register-admin', [AdminController::class, 'create']);
            Route::post('delete-admin', [AdminController::class, 'delete']);
            Route::post('edit-admin', [AdminController::class, 'edit']);


            //controlling the endPoints operations
            Route::get('get-all-endpoints', [EndPointController::class, 'getAll']);
            Route::get('get-endpoint', [EndPointController::class, 'get']);
            Route::post('register-endPoint', [EndPointController::class, 'create']);
            Route::post('edit-register-endPoint', [EndPointController::class, 'edit']);
            Route::post('delete-endPoint', [EndPointController::class, 'delete']);


            //controlling the users operations
            Route::get('get-all-users', [UserController::class, 'getAll']);
            Route::get('get-user', [UserController::class, 'get']);
            Route::post('register-user', [UserController::class, 'create']);
            Route::post('edit-user', [UserController::class, 'edit']);
            Route::post('delete-user', [UserController::class, 'delete']);


            //Controlling the payment Transactions
            Route::get('get-all-payment-transaction', [PaymentTransactionsController::class, 'getAll']);
            Route::get('get-payment-transaction', [PaymentTransactionsController::class, 'get']);
            Route::get('get-endpoint-transactions', [PaymentTransactionsController::class, 'getEndPointTransactions']);
            Route::get('get-endpoint-total-transactions', [PaymentTransactionsController::class, 'getEndPointTotalTransactions']);
            Route::post('create-payment-transaction', [PaymentTransactionsController::class, 'create']);
            Route::post('seen-payment-transaction', [PaymentTransactionsController::class, 'seen']);

            //Controlling the collections Transactions
            Route::get('get-all-collections', [CollectionController::class, 'getAll']);
            Route::get('get-collection', [CollectionController::class, 'get']);
            Route::get('get-endpoint-collections', [CollectionController::class, 'getEndpointCollections']);
            Route::post('create-collection', [CollectionController::class, 'create']);


            //Controlling the Subscriptions operations
            Route::get('get-all-subscription', [SubscriptionController::class, 'getAll']);
            Route::get('get-subscription', [SubscriptionController::class, 'get']);
            Route::post('create-subscription', [SubscriptionController::class, 'create']);
            Route::post('delete-subscription', [SubscriptionController::class, 'delete']);
            Route::post('update-subscription-state', [SubscriptionController::class, 'updateState']);
            Route::post('update-subscription-renual', [SubscriptionController::class, 'updateRenual']);
            Route::post('renew-subscription', [SubscriptionController::class, 'renewSubscription']);
            Route::post('seen-subscription', [SubscriptionController::class, 'seen']);
            Route::post('edit-subscription', [SubscriptionController::class, 'edit']);

            Route::post('extend-subscription', [ExtensionContreoller::class, 'create']);


            //controlling wallet transactions
            Route::post('create-wallet', [WalletController::class, 'create']);
            Route::post('get-wallet-balance', [WalletController::class, 'getWalletBalance']);
            Route::post('charge-wallet-balance', [WalletController::class, 'charge']);


            //general operations
            Route::get('get-tables-count', [AdminController::class, 'tableCounts']);

            //subscripton logs
            Route::get('get-all-subscriptionLog', [SubscriptionLogController::class, 'getAll']);
        });
    });


    /////////////////// EndPoint Apis
    Route::group(['prefix' => 'endPoint'], function () {

        Route::post('login', [AuthController::class, 'login'])->middleware('AssignGuard:endpoints');
        Route::post('logout', [AuthController::class, 'logout'])->middleware('AssignGuard:endpoints');

        Route::group(['middleware' => ['AssignGuard:endpoints', 'CheckValidToken']], function () {

            //controlling the users operations
            Route::get('get-user', [UserController::class, 'get']);
            Route::get('get-users', [UserController::class, 'getBySearch']);

            //Controlling the payment Transactions
            Route::get('get-endpoint-transactions', [PaymentTransactionsController::class, 'getEndPointTransactions']);
            Route::get('get-endpoint-total-transactions', [PaymentTransactionsController::class, 'getEndPointTotalTransactions']);
            Route::post('create-payment-transaction', [PaymentTransactionsController::class, 'create']);

            //Controlling the collections Transactions
            Route::get('get-endpoint-collections', [CollectionController::class, 'getEndpointCollections']);


            //Controlling the Subscriptions operations
            Route::get('get-subscription', [SubscriptionController::class, 'get']);


            //controlling wallet transactions
            Route::post('get-wallet-balance', [WalletController::class, 'getWalletBalance']);
            Route::post('charge-wallet-balance', [WalletController::class, 'charge']);


            //general operations
            Route::get('get-tables-count', [AdminController::class, 'tableCounts']);
        });
    });


    //////////////////
    Route::post('valid', [AdminController::class, 'valid']);
    Route::post('update-valid', [AdminController::class, 'updateValid']);
});
